"""This module is the entry point for the package. It is executed when the package is run as a script."""  # noqa: E501

import argparse

from pyproject_deplister.utils import (
    Args,
    print_output,
    read_pyproject_dependencies,
    save_output,
)

parser = argparse.ArgumentParser(
    description="A package to list the dependencies from a pyproject.toml file.",
)
parser.add_argument(
    "--extra",
    "-e",
    type=str,
    help="The list of extra dependencies.",
    default="",
    nargs="*",
)
parser.add_argument(
    "--path",
    "-p",
    type=str,
    help="The path to the pyproject.toml file.",
    default="pyproject.toml",
    nargs="?",
)
parser.add_argument(
    "--output_file",
    "-o",
    type=str,
    help="The path to the output file, if not provided, the output will be printed to the console.",  # noqa: E501
    nargs="?",
)


def main() -> None:
    """The main function that executes the package."""
    args: Args = parser.parse_args()
    pyproject_content = read_pyproject_dependencies(args)
    if args.output_file and len(args.output_file) > 0:
        save_output(pyproject_content, args.output_file)
    else:
        print_output(pyproject_content)


if __name__ == "__main__":
    main()



"""Tests for pyproject_deplister.py."""

from pathlib import Path

from pyproject_deplister.utils import Args, read_pyproject_dependencies

HERE = Path(__file__).parent


def test_read_pyproject_with_no_dependencies() -> None:
    """Test reading a pyproject.toml file with no dependencies."""
    input_file = HERE / "sample_pyproject_1.toml"
    args = Args(extra=[], path=str(input_file))

    output = read_pyproject_dependencies(args)

    # there should be no dependencies
    assert len(output.dependencies) == 0, "There should be no dependencies"

    # there should be no extra dependencies
    assert len(output.extra_dependencies) == 0, "There should be no extra dependencies"


def test_read_project_with_dependencies() -> None:
    """Test reading a pyproject.toml file with dependencies."""
    input_file = HERE / "sample_pyproject_2.toml"
    args = Args(extra=[], path=str(input_file))

    expected_dependencies = ["pandas", "numpy==1.21.0"]

    output = read_pyproject_dependencies(args)

    # there should be as many dependencies as expected
    assert len(output.dependencies) == len(
        expected_dependencies,
    ), "There should be 2 dependencies"

    # the dependencies should be as expected
    assert output.dependencies == expected_dependencies, "Dependencies should match"


def test_read_project_with_extra_dependencies() -> None:
    """Test reading a pyproject.toml file with extra dependencies."""
    input_file = HERE / "sample_pyproject_3.toml"

    expected_dependencies = ["pandas", "numpy==1.21.0"]
    expected_extra_dependencies = {"dev": ["jupyter"], "test": ["pytest==6.2.4"]}

    args_empty = Args(extra=[], path=str(input_file))
    args_dev = Args(extra=["dev"], path=str(input_file))
    args_test = Args(extra=["test"], path=str(input_file))
    args_all = Args(extra=["dev", "test"], path=str(input_file))

    output_empty = read_pyproject_dependencies(args_empty)

    # there should be as many dependencies as expected
    assert len(output_empty.dependencies) == len(
        expected_dependencies,
    ), "There should be 2 dependencies"

    # the dependencies should be as expected
    assert (
        output_empty.dependencies == expected_dependencies
    ), "Dependencies should match"

    # there should be no extra dependencies
    assert (
        len(output_empty.extra_dependencies) == 0
    ), "There should be no extra dependencies"

    output_dev = read_pyproject_dependencies(args_dev)

    # there should be as many dependencies as expected
    assert len(output_dev.dependencies) == len(
        expected_dependencies,
    ), "There should be 2 dependencies"

    # the dependencies should be as expected
    assert output_dev.dependencies == expected_dependencies, "Dependencies should match"

    # there should be as many extra dependencies as expected
    assert len(output_dev.extra_dependencies) == len(
        expected_extra_dependencies["dev"],
    ), "There should be 1 extra dependency"

    # the extra dependencies should be as expected
    assert (
        output_dev.extra_dependencies["dev"] == expected_extra_dependencies["dev"]
    ), "Extra dependencies should match"

    output_test = read_pyproject_dependencies(args_test)

    # there should be as many dependencies as expected
    assert len(output_test.dependencies) == len(
        expected_dependencies,
    ), "There should be 2 dependencies"

    # the dependencies should be as expected
    assert (
        output_test.dependencies == expected_dependencies
    ), "Dependencies should match"

    # there should be as many extra dependencies as expected
    assert len(output_test.extra_dependencies) == len(
        expected_extra_dependencies["test"],
    ), "There should be 1 extra dependency"

    # the extra dependencies should be as expected
    assert (
        output_test.extra_dependencies["test"] == expected_extra_dependencies["test"]
    ), "Extra dependencies should match"

    output_all = read_pyproject_dependencies(args_all)

    # there should be as many dependencies as expected
    assert len(output_all.dependencies) == len(
        expected_dependencies,
    ), "There should be 2 dependencies"

    # the dependencies should be as expected
    assert output_all.dependencies == expected_dependencies, "Dependencies should match"

    # there should be as many extra dependencies as expected
    assert len(output_all.extra_dependencies) == len(
        expected_extra_dependencies["dev"] + expected_extra_dependencies["test"],
    ), "There should be 2 extra dependencies"

    # the extra dependencies should be as expected
    assert (
        output_all.extra_dependencies["dev"] == expected_extra_dependencies["dev"]
    ), "Extra dependencies should match"

    assert (
        output_all.extra_dependencies["test"] == expected_extra_dependencies["test"]
    ), "Extra dependencies should match"



"""Code logic for the pyproject_deplister module."""

from __future__ import annotations

from dataclasses import dataclass
from typing import Dict, List, Optional

import tomli


@dataclass
class Args:
    """Input arguments for the program."""

    extra: List[str]
    path: str
    output_file: Optional[str]


@dataclass
class Output:
    """Output of the extraction function."""

    dependencies: List[str]
    extra_dependencies: Dict[str, List[str]]

    def __str__(self: Output) -> str:
        """Follows the format of requirements.txt."""
        output = "\n".join(self.dependencies)
        for dependencies in self.extra_dependencies.values():
            output += "\n" + "\n".join(dependencies)
        return output


def read_pyproject_dependencies(args: Args) -> Output:
    """Reads the pyproject.toml file and returns it as a dictionary.

    Args:
        args (Args): The input arguments.

    Returns:
        Output: The output of the function.
    """
    with open(args.path, "rb") as f:
        toml_dict = tomli.load(f)

    return Output(
        dependencies=toml_dict["project"].get("dependencies", []),
        extra_dependencies={
            k: v
            for k, v in toml_dict["project"].get("optional-dependencies", {}).items()
            if k in args.extra
        },
    )


def save_output(output: Output, path: str) -> None:
    """Saves the output of the program to a file.

    Follows the format of requirements.txt.

    Args:
        output (Dict[str, List[str]]): The output of the program.
        path (str): The path to the file.
    """
    with open(path, "w") as f:
        f.write(str(output))


def print_output(output: Output) -> None:
    """Prints the output of the program.

    Follows the format of requirements.txt.

    Args:
        output (Dict[str, List[str]]): The output of the program.
    """
    print(output)  # noqa: T201
